// main.cpp
// by Geoffrey White
// see charlib.h for more info

#include "charlib.h"
#include <sstream>
#include <assert.h>

#define GRID_WIDTH (56) // 1.4 : 1
#define GRID_HEIGHT (40)

static void draw_main()
{
	char_style style;
	int x, y;

	for (x = 0; x < GRID_WIDTH; x++)
	{
		for (y = 0; y < GRID_HEIGHT; y++)
		{
			if ((x == 0) || (x == GRID_WIDTH - 1) ||
				(y == 0) || (y == GRID_HEIGHT - 1))
			{
				char_draw(x, y, '#', char_default_style);
			} else {
				char_draw(x, y, '.', char_default_style);
			}
		}
	}

	char_draw(2, 2, "@", char_default_style);
	char_draw(4, 2, "Hello, world!", char_default_style);

	for (x = 0; x < 16; x++)
	{
		for (y = 0; y < 6; y++)
		{
			char_draw(4 + x, 8 + y, x + (y * 16) + 32, char_default_style);
		}
	}

	for (x = 0; x < 16; x++)
	{
		for (y = 0; y < 8; y++)
		{
			style.fore_r = (rand() & 0xFF);
			style.fore_g = (rand() & 0xFF);
			style.fore_b = (rand() & 0xFF);
			style.back_r = (rand() & 0xFF);
			style.back_g = (rand() & 0xFF);
			style.back_b = (rand() & 0xFF);
			char_draw(4 + x, 15 + y, '*', style);
		}
	}
}

static void draw_key(const char_key &key)
{
	std::stringstream stmp1, stmp2;

	stmp1 << int(key.ascii);
	stmp2 << int(key.sdl);

	char_draw(4, 4, std::string("key    ") + key.ascii, char_default_style);
	char_draw(4, 5, std::string("ascii  ") + stmp1.str() + std::string("...."), char_default_style);
	char_draw(4, 6, std::string("SDLKey ") + stmp2.str() + std::string("...."), char_default_style);
}

int main(int argc, char *argv[])
{
	char_config config;

	// config
	config.window_title = "CharLib Demo";
	config.icon_file_name = "Data\\icon32.bmp";
	config.ascii_decl.file_name = "Data\\tiles_ascii.bmp"; // ascii characters 32 .. 127
	config.ascii_decl.grid_width = 8;
	config.ascii_decl.grid_height = 12;
	config.grid_width = GRID_WIDTH;
	config.grid_height = GRID_HEIGHT;
	config.aspect_width = 1;
	config.aspect_height = 1;

#ifndef _DEBUG
	try
#endif
	{
		char_key key;

		// --- init charlib ---

		char_init(config);

		// --- run game ---

		draw_main();
		while (char_update())
		{
			// don't hog the CPU
			SDL_Delay(20);

			// get input
			if (char_get_key(key))
			{
				draw_key(key);
			}
		}
#ifndef _DEBUG
	} catch (const std::exception &e) {
		char_display_error(std::string("An error has occurred:\n") +
						   std::string(e.what()) +
						   std::string("\n\n'") +
						   config.window_title +
						   std::string("' is shutting down."));
#endif
	}

	// --- shutdown ---

	char_shutdown();

	return 0;
}
